/* ****************************************************************************
 *  CodeSketch.cpp
 *  Copyright (C) 2002, John F. Hubbard
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License (LGPL) along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  As this software is very much in use and evolving, you may want to check
 *  this web site for periodic updates: http://www.hubbard-software.com
 *  You may also post feedback, comments, and (someday) patches there.
 *
 *  My goal is to provide quality, open source programmer tools that are
 *  available to everyone--including commercial users. This is intended to help
 *  the software profession, by providing quality tools that cannot be taken
 *  away from you, no matter where you work.  The LGPL license seems to allow
 *  for all of this, which is why I use it.
 *  --jfh, December, 2002.
 *
 *  $Revision: 1.11 $
 *  $Date: 2002/12/15 19:45:07 $
 *  $Author: jhubbard $
 *************************************************************************** */
#include <string>
#include <iostream>
#include "CodeSketch.h"
#include "GeneratorBase.h"
#include "Guidelines.h"
#include "AnsiCGeneration.h"
#include "CplusplusGeneration.h"
#include "DeviceDriverGeneration.h"


namespace code_sketch
{

CodeSketch::CodeSketch( const Guidelines& guidelines )
	:	mGuidelines( guidelines )
{

}

CodeSketch::~CodeSketch()
{
}

void CodeSketch::Run()
{
	if( mGuidelines.ModeIsAnsiC() )
	{
		AnsiCBodyGenerator  body(mGuidelines);
		AnsiCHeaderGenerator  header(mGuidelines);
		GenerateFile( body );
		GenerateFile( header );
	}
	else if( mGuidelines.ModeIsCplusplus() )
	{
		ClassHeaderGenerator  header(mGuidelines);
		GenerateFile( header );

		if(! mGuidelines.PureInterfaceClassDesired()  )
		{
			ClassBodyGenerator  body(mGuidelines);
			GenerateFile( body );

		}

		if(    mGuidelines.InlineFileDesired()
			&& ! mGuidelines.PureInterfaceClassDesired() )
		{
			ClassInlineGenerator  inlineFile(mGuidelines);
			GenerateFile( inlineFile );
		}
	}
	else if( mGuidelines.ModeIsDeviceDriver() )
	{
		if( mGuidelines.TargetOperatingSystem() == "linux" )
		{
			Linux_x86_DeviceDriverGenerator  deviceDriver(mGuidelines);
			GenerateFile( deviceDriver );
		}
		else if( mGuidelines.TargetOperatingSystem() == "lynx")
		{
			LynxOS_ppc_DeviceDriverGenerator  deviceDriver(mGuidelines);
			GenerateFile( deviceDriver );
			
			LynxOS_ppc_ImportsGenerator  importFile(mGuidelines);
			GenerateFile( importFile );
			
			LynxOS_InfoMainGenerator  infoMain(mGuidelines);
			GenerateFile( infoMain );
			
			LynxOS_InfoHeaderGenerator  infoHeader(mGuidelines);
			GenerateFile( infoHeader );
		}

		DeviceDriverBuildScriptGenerator buildScript(mGuidelines);
		GenerateFile( buildScript );
		
		if( !mGuidelines.SparseCodeGenerationDesired() )
		{
			CommonBuildScriptGenerator commonBuildScript(mGuidelines);
			GenerateFile( commonBuildScript );
			
			CommonDeviceDriverRoutinesGenerator commonDDRoutines(mGuidelines);
			GenerateFile( commonDDRoutines );
		}
	}

	// Common options:
	if( mGuidelines.UnitTestDesired()
		// A separate, stand-alone program is used to test device drivers.
		// This program, called CodeSketchDeviceController, is supplied
		// with the CodeSketch generator.  Thus, CodeSketch does not generate
		// a unit test file when in device driver mode.
		&& ! mGuidelines.ModeIsDeviceDriver())
	{
		UnitTestGenerator  unitTest(mGuidelines);
		GenerateFile( unitTest );
	}
	if( mGuidelines.BuildScriptDesired() )
	{
		MakefileGenerator  buildScript(mGuidelines);
		GenerateFile( buildScript );
	}
}

void CodeSketch::DumpDiagnostics() const
{
//     mGuidelines.DumpDiagnostics();
}

void CodeSketch::GenerateFile(GeneratorBase & arGeneratable)
{
	arGeneratable.CreateHeader();
	arGeneratable.CreateProlog();

	for( std::vector<std::string>::iterator it = mGuidelines.ClassList().begin();
		 it != mGuidelines.ClassList().end();
		 ++it)
	{
		arGeneratable.CreateBody(*it);
	}
	arGeneratable.CreateEpilog();
}


} // end of the code_sketch namespace
