/* ****************************************************************************
 *  CodeSketchDeviceController.cpp
 *  Copyright (C) 2002, John F. Hubbard
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License (LGPL) along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  As this software is very much in use and evolving, you may want to check
 *  this web site for periodic updates: http://www.hubbard-software.com
 *  You may also post feedback, comments, and (someday) patches there.
 *
 *  My goal is to provide quality, open source programmer tools that are
 *  available to everyone--including commercial users. This is intended to help
 *  the software profession, by providing quality tools that cannot be taken
 *  away from you, no matter where you work.  The LGPL license seems to allow
 *  for all of this, which is why I use it.
 *  --jfh, December, 2002.
 *
 *  $Revision: 1.10 $
 *  $Date: 2002/12/17 11:17:53 $
 *  $Author: jhubbard $
 *************************************************************************** */

#include <sys/ioctl.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <string>
#include <iostream>
#include "CommandLine.h"
#include "StandardCodeSketchCommands.h"

namespace code_sketch
{
	void IoControl( const tools::CommandLine & arCommands);
	void Usage();
	std::string Version();
}

int main(int argc, char* argv[])
{
	try
	{
		using namespace tools;
		CommandLine commands(argc, argv);
		if(     commands.Exists("version")
			||  commands.Exists("version"))
		{
			std::cout << code_sketch::Version() << std::endl;
			return 0;
		}

		if(     commands.Exists("help")
			||  commands.Exists("help")
			||  commands.Exists("h")
			||  commands.Count() == 0   )
		{
			code_sketch::Usage();
			return 0;
		}

		code_sketch::IoControl( commands );
	}
	catch(std::string& errorMessage)
	{
		std::cout << "FAILED: " << errorMessage << std::endl;
		std::cout << "For usage information, please run this program with the -help option."
				  << std::endl;
		return 1;
	}
    return 0;
}
/////////////////////////////////////////////////////////////////////////////////////
namespace code_sketch
{
using std::endl;

void IoControl( const tools::CommandLine & arCommands)
{
	std::string deviceName;
	if( arCommands.Exists("dev") )
	{
		deviceName = arCommands.GetByName("dev");
	}
	else if( arCommands.Exists("device") )
	{
		deviceName = arCommands.GetByName("device");
	}
	else
	{
		throw (std::string("Missing -dev or --device parameter."));
	}

	StandardCodeSketchCommands  ioCommands;
	memset(&ioCommands, 0, sizeof(ioCommands));

	int fd = open(const_cast<char*>(deviceName.c_str()), O_RDWR);
	if( -1 == fd )
	{
		perror("Failed to open the device node");
		exit(2);
	}

	if( arCommands.Exists("dump") )
	{
		ioCommands.mCommand = CODE_SKETCH_COMMAND_DUMP;
		// Run the command:
		ioctl(fd, CODE_SKETCH_OVERALL_COMMAND_TO_IOCTL, &ioCommands);
	}
	else if( arCommands.Exists("seek-forward") )
	{
		ioCommands.mCommand = CODE_SKETCH_COMMAND_SEEK_FORWARD;
		ioCommands.mDataIn  = atol(arCommands.GetByName("seek-forward").c_str());
		lseek(fd, ioCommands.mDataIn, SEEK_CUR);
	}
	else if( arCommands.Exists("sf") )
	{
		ioCommands.mCommand = CODE_SKETCH_COMMAND_SEEK_FORWARD;
		ioCommands.mDataIn  = atol(arCommands.GetByName("sf").c_str());
		lseek(fd, ioCommands.mDataIn, SEEK_CUR);
	}
	else if( arCommands.Exists("seek-back") )
	{
		ioCommands.mCommand = CODE_SKETCH_COMMAND_SEEK_BACK;
		ioCommands.mDataIn  = atol(arCommands.GetByName("seek-back").c_str());
		lseek(fd, 0 - ioCommands.mDataIn, SEEK_CUR);
	}
	else if( arCommands.Exists("sb") )
	{
		ioCommands.mCommand = CODE_SKETCH_COMMAND_SEEK_BACK;
		ioCommands.mDataIn  = atol(arCommands.GetByName("sb").c_str());
		lseek(fd, 0 - ioCommands.mDataIn, SEEK_CUR);
	}
	else
	{
		throw (std::string("Unrecognized or missing command."));
	}

	if( ioCommands.mBufferOut != 0 )
	{
		std::cout << "Data returned: " << endl << ioCommands.mBufferOut << std::endl;
	}
	
	if( -1 == close(fd) )
	{
		perror("Failed to close the device");
		exit(3);
	}
}


std::string Version()
{
	return "This is IoControl, version 1.02, created on 14 December, 2002.";
}

void Usage()
{
	using std::endl;
	std::cout <<
		"----------------------------------------------------------------------"	<< endl <<
		Version() 																	<< endl <<
		"Assuming that you generated the device driver with the CodeSketch"			<< endl <<
		"program..."																<< endl <<
		"" 																			<< endl <<
		"Usage: (Assuming that you generated the device driver with CodeSketch)"	<< endl <<
		"       IoControl [-dev | --device] device [options]"						<< endl <<
		"Options:"																	<< endl <<
		"    [ -help | -help | -h ]" 												<< endl <<
		"    [ -version | -version ]"			 									<< endl <<
		"    -dump" 																<< endl <<
		"    [ -seek-forward | -sf] distance  " 									<< endl <<
 		"    [ -seek-back    | -sb] distance  " 									<< endl <<
		"" 																			<< endl <<
		"Examples: " 																<< endl <<
		"  1. This moves the file pointer AHEAD one character position:" 			<< endl <<
		"  ControlFullDriver -dev /dev/yourDriver --seek-forward 1" 				<< endl <<
		"" 																			<< endl <<
		"  2. This moves the file pointer BACK ten character positions:" 			<< endl <<
		"  ControlFullDriver -dev /dev/yourDriver -sb 10" 							<< endl <<
		"" 																			<< endl <<
		"  3. This example prints this screen:" 			  					  	<< endl <<
		"  ControlFullDriver -help" 												<< endl <<
		"" 																			<< endl <<
		"  4. This example dumps the internal driver state to standard output:"		<< endl <<
		"  ControlFullDriver -dump"						 							<< endl <<
		"" 																			<< endl <<
		"Fixes and updates are maintained at http://www.hubbard-software.com" 		<< endl <<
		"----------------------------------------------------------------------"	<< endl <<
		"" 																			<< endl;
}


} // end of namespace code_sketch
