/* ****************************************************************************
 *  CplusplusGeneration.h
 *  Copyright (C) 2002, John F. Hubbard
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License (LGPL) along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  As this software is very much in use and evolving, you may want to check
 *  this web site for periodic updates: http://www.hubbard-software.com
 *  You may also post feedback, comments, and (someday) patches there.
 *
 *  My goal is to provide quality, open source programmer tools that are
 *  available to everyone--including commercial users. This is intended to help
 *  the software profession, by providing quality tools that cannot be taken
 *  away from you, no matter where you work.  The LGPL license seems to allow
 *  for all of this, which is why I use it.
 *  --jfh, December, 2002.
 *
 *  $Revision: 1.7 $
 *  $Date: 2002/12/15 01:06:24 $
 *  $Author: jhubbard $
 *************************************************************************** */
#ifndef JFH_CODE_SKETCH_CPLUSPLUSGENERATION_H
#define JFH_CODE_SKETCH_CPLUSPLUSGENERATION_H

#include "GeneratorBase.h"
#include "FileCreator.h"
#include <fstream>

namespace code_sketch
{
	 class Guidelines; // Forward declaration

/** Purpose: Generator a header file (typically a *.h file).
 */
class ClassHeaderGenerator : public GeneratorBase
{
public:
    /// Constructor.
    ClassHeaderGenerator( const Guidelines& arGuidelines );
    virtual ~ClassHeaderGenerator();
	virtual void CreateProlog();
	virtual void CreateBody(const std::string& arClassName) ;
	virtual void CreateEpilog();
	virtual FileCreator& GetFileCreator() {return mFileCreator;};
private:
	// Copying or assignment of this class is prohibited:
    ClassHeaderGenerator(const ClassHeaderGenerator&);
    ClassHeaderGenerator& operator=(const ClassHeaderGenerator&);

	FileCreator		mFileCreator;
};
/** Purpose: Generate a file to contain inline definitions and
	template instantiations.  Such a file is included from the bottom
	of the header file.  This helps provide at least the appearance
	of a separation between implementation and interface, and also
	greatly eases the chore of switching between inline and non-inline
	versions of the same method.
 */
class ClassInlineGenerator : public GeneratorBase
{
public:
    /// Constructor.
    ClassInlineGenerator( const Guidelines& arGuidelines );
    virtual ~ClassInlineGenerator();
	virtual void CreateProlog();
	virtual void CreateBody(const std::string& arClassName) ;
	virtual void CreateEpilog();
	virtual FileCreator& GetFileCreator() {return mFileCreator;};
private:
	// Copying or assignment of this class is prohibited:
    ClassInlineGenerator(const ClassInlineGenerator&);
    ClassInlineGenerator& operator=(const ClassInlineGenerator&);

	FileCreator		mFileCreator;
};
/** Purpose: Generate the implementation file for a C++ class, or the
	implementation file for a set of C routines (depending upon whether
	C or C++ code generation is occuring).
 */
class ClassBodyGenerator : public GeneratorBase
{
public:
    /// Constructor.
    ClassBodyGenerator( const Guidelines& arGuidelines );
    virtual ~ClassBodyGenerator();
	virtual void CreateProlog();
	virtual void CreateBody(const std::string& arClassName) ;
	virtual void CreateEpilog();
	virtual FileCreator& GetFileCreator() {return mFileCreator;};
private:
	// Copying or assignment of this class is prohibited:
    ClassBodyGenerator(const ClassBodyGenerator&);
    ClassBodyGenerator& operator=(const ClassBodyGenerator&);

	FileCreator		mFileCreator;
};
/** Purpose: Generate a file that contains the main() routine, in order
	to instantiate and exercise generated classes (in the case of C++
	generation), or to invoke routines (in the case of C generation).

	UnitTestGenerator is NOT instantiated in the case of device driver generation,
	because a separate, stand-alone program is provided for conducting
	ioctl-based testing.  This program is called IoControl, and the source
	code for it is in the file CodeSketchDeviceController.cpp
 */
class UnitTestGenerator : public GeneratorBase
{
public:
    /// Constructor.
    UnitTestGenerator( const Guidelines& arGuidelines );
    virtual ~UnitTestGenerator();
	virtual void CreateProlog();
	virtual void CreateBody(const std::string& arClassName) ;
	virtual void CreateEpilog();
	virtual FileCreator& GetFileCreator() {return mFileCreator;};
private:
	// Copying or assignment of this class is prohibited:
    UnitTestGenerator(const UnitTestGenerator&);
    UnitTestGenerator& operator=(const UnitTestGenerator&);

	FileCreator		mFileCreator;
};
/** Purpose: Generate a build script.  For C and C++ mode, this usually
    means a makefile.  For device drivers, it is often a bash script, because
	building device drivers can be intricate enough to warrant the extra
	power of a full scripting language.
 */
class MakefileGenerator : public GeneratorBase
{
public:
    /// Constructor.
    MakefileGenerator( const Guidelines& arGuidelines );
    virtual ~MakefileGenerator();
	virtual void CreateHeader();
	virtual void CreateProlog();
	virtual FileCreator& GetFileCreator() {return mFileCreator;};
private:
	// Copying or assignment of this class is prohibited:
    MakefileGenerator(const MakefileGenerator&);
    MakefileGenerator& operator=(const MakefileGenerator&);

	FileCreator		mFileCreator;
};
} // end of the code_sketch namespace

#endif // #ifndef JFH_CODE_SKETCH_CPLUSPLUSGENERATION_H

