/* ****************************************************************************
 *  DeviceDriverGeneration.h
 *  Copyright (C) 2002, John F. Hubbard
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License (LGPL) along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  As this software is very much in use and evolving, you may want to check
 *  this web site for periodic updates: http://www.hubbard-software.com
 *  You may also post feedback, comments, and (someday) patches there.
 *
 *  My goal is to provide quality, open source programmer tools that are
 *  available to everyone--including commercial users. This is intended to help
 *  the software profession, by providing quality tools that cannot be taken
 *  away from you, no matter where you work.  The LGPL license seems to allow
 *  for all of this, which is why I use it.
 *  --jfh, December, 2002.
 *
 *  $Revision: 1.8 $
 *  $Date: 2002/12/15 05:03:19 $
 *  $Author: jhubbard $
 *************************************************************************** */
#ifndef JFH_CODE_SKETCH_DEVICEDRIVERGENERATION_H
#define JFH_CODE_SKETCH_DEVICEDRIVERGENERATION_H

#include "GeneratorBase.h"
#include "FileCreator.h"

namespace code_sketch
{
/** Purpose: Generate the file that contains the entry points and full
	implementation code for a Linux/x86 device driver.
 */
class Linux_x86_DeviceDriverGenerator : public GeneratorBase
{
public:
    /// Constructor.
    Linux_x86_DeviceDriverGenerator( const Guidelines& arGuidelines );
    virtual ~Linux_x86_DeviceDriverGenerator();
	virtual void CreateBody(const std::string& arClassName);
	virtual FileCreator& GetFileCreator() {return mFileCreator;};
private:
	// Copying or assignment of this class is prohibited:
    Linux_x86_DeviceDriverGenerator(const Linux_x86_DeviceDriverGenerator&);
    Linux_x86_DeviceDriverGenerator& operator=(const Linux_x86_DeviceDriverGenerator&);

	FileCreator		mFileCreator;
};
/** Purpose: Generate the file that contains the entry points and full
	implementation code for a LynxOS/PowerPC device driver.
 */
class LynxOS_ppc_DeviceDriverGenerator : public GeneratorBase
{
public:
    /// Constructor.
    LynxOS_ppc_DeviceDriverGenerator( const Guidelines& arGuidelines );
    virtual ~LynxOS_ppc_DeviceDriverGenerator();
	virtual void CreateBody(const std::string& arClassName) ;
	virtual FileCreator& GetFileCreator() {return mFileCreator;};
private:
	// Copying or assignment of this class is prohibited:
    LynxOS_ppc_DeviceDriverGenerator(const LynxOS_ppc_DeviceDriverGenerator&);
    LynxOS_ppc_DeviceDriverGenerator& operator=(const LynxOS_ppc_DeviceDriverGenerator&);

	FileCreator		mFileCreator;
};

/** Purpose: Generate the file that contains the "info" declarations
	for a LynxOS device driver.
 */
class LynxOS_InfoHeaderGenerator : public GeneratorBase
{
public:
    /// Constructor.
    LynxOS_InfoHeaderGenerator( const Guidelines& arGuidelines );
    virtual ~LynxOS_InfoHeaderGenerator();
	virtual void CreateBody(const std::string& arClassName) ;
	virtual FileCreator& GetFileCreator() {return mFileCreator;};
private:
	// Copying or assignment of this class is prohibited:
    LynxOS_InfoHeaderGenerator(const LynxOS_InfoHeaderGenerator&);
    LynxOS_InfoHeaderGenerator& operator=(const LynxOS_InfoHeaderGenerator&);

	FileCreator		mFileCreator;
};

/** Purpose: Generate the file a program (intended to be run on a LynxOS
	computer) that can create the binary "info" file, needed by a LynxOS
	device driver.
 */
class LynxOS_InfoMainGenerator : public GeneratorBase
{
public:
    /// Constructor.
    LynxOS_InfoMainGenerator( const Guidelines& arGuidelines );
    virtual ~LynxOS_InfoMainGenerator();
	virtual void CreateBody(const std::string& arClassName) ;
	virtual FileCreator& GetFileCreator() {return mFileCreator;};
private:
	// Copying or assignment of this class is prohibited:
    LynxOS_InfoMainGenerator(const LynxOS_InfoMainGenerator&);
    LynxOS_InfoMainGenerator& operator=(const LynxOS_InfoMainGenerator&);

	FileCreator		mFileCreator;
};

/** Purpose: Generate the file that contains the imports
	for a LynxOS/PowerPC device driver.
 */
class LynxOS_ppc_ImportsGenerator : public GeneratorBase
{
public:
    /// Constructor.
    LynxOS_ppc_ImportsGenerator( const Guidelines& arGuidelines );
    virtual ~LynxOS_ppc_ImportsGenerator();
	virtual void CreateHeader();
	virtual void CreateProlog();
	virtual void CreateBody(const std::string& arClassName);
	virtual void CreateEpilog();
	virtual FileCreator& GetFileCreator() {return mFileCreator;};
private:
	// Copying or assignment of this class is prohibited:
    LynxOS_ppc_ImportsGenerator(const LynxOS_ppc_ImportsGenerator&);
    LynxOS_ppc_ImportsGenerator& operator=(const LynxOS_ppc_ImportsGenerator&);

	FileCreator		mFileCreator;
};

/** Purpose: Generate a file to hold the routines common to more than one
	piece of a device driver.
 */
class CommonDeviceDriverRoutinesGenerator : public GeneratorBase
{
public:
    /// Constructor.
    CommonDeviceDriverRoutinesGenerator( const Guidelines& arGuidelines );
    virtual ~CommonDeviceDriverRoutinesGenerator();
	virtual void CreateProlog();
	virtual void CreateBody(const std::string& arClassName) ;
	virtual void CreateEpilog();
	virtual FileCreator& GetFileCreator() {return mFileCreator;};
private:
	// Copying or assignment of this class is prohibited:
    CommonDeviceDriverRoutinesGenerator(const CommonDeviceDriverRoutinesGenerator&);
    CommonDeviceDriverRoutinesGenerator& operator=(const CommonDeviceDriverRoutinesGenerator&);

	FileCreator		mFileCreator;
};
/** Purpose: Generate a file that can build a device driver.
 */
class DeviceDriverBuildScriptGenerator : public GeneratorBase
{
public:
    /// Constructor.
    DeviceDriverBuildScriptGenerator( const Guidelines& arGuidelines );
    virtual ~DeviceDriverBuildScriptGenerator();
	virtual void CreateHeader();
	virtual void CreateProlog();
	virtual FileCreator& GetFileCreator() {return mFileCreator;};
private:
	// Copying or assignment of this class is prohibited:
    DeviceDriverBuildScriptGenerator(const DeviceDriverBuildScriptGenerator&);
    DeviceDriverBuildScriptGenerator& operator=(const DeviceDriverBuildScriptGenerator&);

	FileCreator		mFileCreator;
};

/** Purpose: Generate a file to hold common build script code.  This can be
	suppressed through a command-line option.
 */
class CommonBuildScriptGenerator : public GeneratorBase
{
public:
    /// Constructor.
    CommonBuildScriptGenerator( const Guidelines& arGuidelines );
    virtual ~CommonBuildScriptGenerator();
	virtual void CreateHeader();
	virtual void CreateProlog();
	virtual FileCreator& GetFileCreator() {return mFileCreator;};
private:
	// Copying or assignment of this class is prohibited:
    CommonBuildScriptGenerator(const CommonBuildScriptGenerator&);
    CommonBuildScriptGenerator& operator=(const CommonBuildScriptGenerator&);

	FileCreator		mFileCreator;
};

} // end of the code_sketch namespace

#endif // #ifndef JFH_CODE_SKETCH_DEVICEDRIVERGENERATION_H

