/* ****************************************************************************
 *  GeneratorBase.cpp
 *  Copyright (C) 2002, John F. Hubbard
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License (LGPL) along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  As this software is very much in use and evolving, you may want to check
 *  this web site for periodic updates: http://www.hubbard-software.com
 *  You may also post feedback, comments, and (someday) patches there.
 *
 *  My goal is to provide quality, open source programmer tools that are
 *  available to everyone--including commercial users. This is intended to help
 *  the software profession, by providing quality tools that cannot be taken
 *  away from you, no matter where you work.  The LGPL license seems to allow
 *  for all of this, which is why I use it.
 *  --jfh, December, 2002.
 *
 *  $Revision: 1.11 $
 *  $Date: 2002/12/15 01:06:24 $
 *  $Author: jhubbard $
 *************************************************************************** */
#include <string>
#include <ctype.h>
#include <iostream>
#include <time.h>
#include "Guidelines.h"
#include "GeneratorBase.h"
#include "FileCreator.h"

namespace code_sketch
{
using std::endl;

GeneratorBase::GeneratorBase( const Guidelines& arGuidelines )
	: mGuidelines( arGuidelines )
{
}

GeneratorBase::~GeneratorBase()
{
}

void
GeneratorBase::CreateHeader()
{
	// This is the default implementation of the virtual method.
	CreateStandard_C_Header( GetFileCreator().FileName(), "" );
}

void GeneratorBase::CreateProlog()
{
	// Default implementation: do not generate anything for the prolog.
}
void GeneratorBase::CreateBody(const std::string& arClassName)
{
	// Default implementation: do not generate anything for the body.
}
void GeneratorBase::CreateEpilog()
{
	// Default implementation: do not generate anything for the epilog.
}


std::ostream& 	
GeneratorBase::SendToFile()
{
	return GetFileCreator().SendToFile();
}

void
GeneratorBase::CreateStandardHeader( const std::string& arFileName,
									 const std::string& arSpecificWords,
									 const std::string& arPrefix)
{
	SendToFile() <<
    arPrefix  << "  " << arFileName													 	<< endl <<
    arPrefix << "  Created by " << GetGuidelines().Author() << ", on " << DateAndTime()	<< endl <<
    arPrefix 	 																		<< endl <<
    arPrefix  << "  " <<GetGuidelines().Copyright() 									<< endl;

	if(GetGuidelines().OpenSourceDesired())
	{
		SendToFile() <<
		arPrefix				  		  												 << endl <<
		arPrefix << "  This library is free software; you can redistribute it and/or" 	 << endl <<
		arPrefix << "  modify it under the terms of the GNU Lesser General Public" 		 << endl <<
		arPrefix << "  License as published by the Free Software Foundation; either" 	 << endl <<
		arPrefix << "  version 2.1 of the License, or (at your option) any later version." << endl <<
		arPrefix << "" 																	 << endl <<
		arPrefix << "  This library is distributed in the hope that it will be useful,"  << endl <<
		arPrefix << "  but WITHOUT ANY WARRANTY; without even the implied warranty of" 	 << endl <<
		arPrefix << "  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU" << endl <<
		arPrefix << "  Lesser General Public License for more details." 				  << endl <<
		arPrefix << "" << endl <<
		arPrefix << "  You should have received a copy of the GNU Lesser General Public"  << endl <<
		arPrefix << "  License (LGPL) along with this library; if not, write to the Free Software" << endl <<
		arPrefix << "  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA" << endl <<
		arPrefix 	 													 					<< endl;
	}

	SendToFile() << arSpecificWords;

	if(GetGuidelines().SccsKeywordsDesired())
	{
		SendToFile() <<
		arPrefix  		  												 << endl <<
		arPrefix << "  $" "Revision: 1.0" " $"		  					 << endl <<
		arPrefix << "  $" "Date: " << DateAndTime() << " $"	  			 << endl <<
		arPrefix << "  $" "Author: " << GetGuidelines().Author() << " $" << endl;
	}
}

void
GeneratorBase::CreateStandard_C_Header( const std::string& arFileName,
										const std::string& arSpecificWords)
{
	SendToFile() <<
    "/*****************************************************************************"	<< endl;

    CreateStandardHeader(arFileName, arSpecificWords, " *");

	SendToFile() <<
    " *************************************************************************** */"	<< endl;
}

void
GeneratorBase::CreateStandardScriptHeader( const std::string& arFileName,
										   const std::string& arSpecificWords)
{
	SendToFile() <<
    "#############################################################################"	<< endl;

	CreateStandardHeader(arFileName, arSpecificWords, "#");

	SendToFile() <<
    "#############################################################################"	<< endl;
}

void
GeneratorBase::CreateStandardMakefile()
{
	SendToFile() <<
	"PROGRAM=" << GetGuidelines().BaseUnitTestName()								<< endl <<
	"HEADERS=" << GetGuidelines().ClassHeaderName()									<< endl <<
	"INCLUDES=" 								 									<< endl <<
	""																			    << endl <<
	"ifeq (\"\",\"$(TARGET)\")"													    << endl <<
	"\t$(warning  Warning: You did not specify a value for TARGET,"
						   " so LINUX_X86 was chosen by default.)"  				<< endl <<
	"\tTARGET=LINUX_X86"														    << endl <<
	"endif"																		 	<< endl <<
		""																		    << endl;

	if( GetGuidelines().ModeIsCplusplus())
	{
		if( GetGuidelines().PureInterfaceClassDesired() )
		{
			// As a way of explaining the mechanics of the makefile, let's generate
			// empty definitions of these two variables, to emphasize that they
			// *should* be empty.
			SendToFile() <<
			"CPP_OBJS=" 	<< endl <<
			"C_OBJS=" 	<< endl;
		}
		else
		{
			SendToFile() <<
			"CPP_OBJS=	    ${TARGET}/"       << GetGuidelines().BaseFilename()  <<".o   " 	<< endl <<
			"C_OBJS=" 	<< endl;
		}
	}
	else if( GetGuidelines().ModeIsAnsiC() )
	{
		SendToFile() <<
		"CPP_OBJS=" 	<< endl <<
		"C_OBJS=	    ${TARGET}/"       << GetGuidelines().BaseFilename()  <<".o   " 	<< endl;
	}

	if( GetGuidelines().UnitTestDesired() )
	{
		SendToFile() <<
		"UNIT_TEST_OBJS=      ${TARGET}/"     << GetGuidelines().BaseUnitTestName() << ".o"	<< endl;
	}
	else
	{
		SendToFile() <<
		"UNIT_TEST_OBJS="	<< endl;
	}
	SendToFile() <<
	""																			    << endl <<
	"LOCAL_CXX:=g++"																<< endl <<
	"LOCAL_CC:=gcc"																	<< endl <<
	"LINK:=g++"																	    << endl <<
	""																			    << endl <<
	"C_FLAGS:=-D${TARGET} -D_REENTRANT -c -O2 -g  \\"		    		 			<< endl <<
	"	   -Wall -Wno-unknown-pragmas -Wno-unused -Werror "							<< endl <<
	""																	  			<< endl <<
	"CXX_FLAGS:=${CXX_FLAGS}"														<< endl <<
	"LINK_FLAGS= "											    					<< endl <<
	""																	  			<< endl <<
	"all: setup $(TARGET)/$(PROGRAM)"									   			<< endl <<
	"\t@echo \"Building for the $(TARGET) platform...\""							<< endl <<
	""																	  			<< endl <<
	"setup:"															    		<< endl <<
	"\t@if [ ! -d $(TARGET) ];then mkdir $(TARGET);fi"					  			<< endl <<
	""																	  			<< endl <<
	"html: ${HEADERS}"												  				<< endl <<
	"\tdoc++ -d html ${HEADERS}"													<< endl <<
	"\ttouch html"													      			<< endl <<
	""																	  			<< endl <<
	"clean:"															    		<< endl <<
	"\t@if [ -d $(TARGET) ];then rm -r $(TARGET);fi;"					   			<< endl <<
	"\t@if [ -d html ];     then rm -r html;     fi;"					   			<< endl <<
	"\t@if [ -d Debug ];    then rm -r Debug;    fi;"					   			<< endl <<
	"\t@if [ -d Release ];  then rm -r Release;  fi;"					   			<< endl <<
	"\t@if [ -f *.plg ];    then rm *.plg;       fi;"					   			<< endl <<
	"\t@if [ -f *.opt ];    then rm *.opt;       fi;"					   			<< endl <<
	"\t@if [ -f *.ncb ];    then rm *.ncb;       fi;"					   			<< endl <<
	"\t@if [ -f *.dsw ];    then rm *.dsw;       fi;"					   			<< endl <<
	""																	  			<< endl <<
	"$(TARGET)/$(PROGRAM): $(C_OBJS) $(CPP_OBJS) $(UNIT_TEST_OBJS)"			       	<< endl <<
	"\t@echo \"Linking $@ ...\""													<< endl <<
	"\t$(LINK) $(LINK_FLAGS) $(C_OBJS) $(CPP_OBJS) $(UNIT_TEST_OBJS) -o $(TARGET)/$(PROGRAM)"							<< endl <<
	""																	  			<< endl <<
	""																	  			<< endl <<
	"$(TARGET)/%.o:%.cpp"											       			<< endl <<
	"\t@echo \"Building $@ ...\""									       			<< endl <<
	"\t$(LOCAL_CXX) $(CXX_FLAGS) -c $< -o $@ $(INCLUDES)"					  			<< endl <<
	""																      			<< endl <<
	"$(TARGET)/%.o:%.c"											       				<< endl <<
	"\t@echo \"Building $@ ...\""									       			<< endl <<
	"\t$(LOCAL_CC) $(C_FLAGS) -c $< -o $@ $(INCLUDES)"					  		   		<< endl <<
	""																      			<< endl <<
	"local_test: all"												       			<< endl <<
	"\t@$(TARGET)/$(PROGRAM)"										       			<< endl <<
	""																      			<< endl <<
	"dev_test: all"													 				<< endl <<
	"\t@$(TARGET)/$(PROGRAM) > $(PROGRAM).out"						      			<< endl <<
	""																      			<< endl <<
	"\t@if cmp -s $(PROGRAM).sig $(PROGRAM).out; then   \\" 						<< endl <<
	"\t\techo \"UnitTest $(PROGRAM):	SUCCESS\";      \\"  						<< endl <<
	"\telif [ -f $(PROGRAM).sig ]; then              \\" 							<< endl <<
	"\t\techo \"UnitTest $(PROGRAM):	FAILED\";           \\"   					<< endl <<
	"\t\tdiff $(PROGRAM).sig $(PROGRAM).out;                \\"	  					<< endl <<
	"\telse                              \\" 										<< endl <<
	"\t\techo \"UnitTest $(PROGRAM):	UNKNOWN (no signature file exists)\";  \\" 	<< endl <<
	"\tfi"															      			<< endl <<
	""																	  			<< endl;
}

std::string
GeneratorBase::MangledUpperCaseName( const Guidelines& arGuidelines )
{
	return  arGuidelines.CustomInitials() + "_" + UpperCase(arGuidelines.Namespace())
		   + std::string("_") + UpperCase(arGuidelines.BaseFilename()) + std::string("_H");
}

std::string
GeneratorBase::DateAndTime()
{
	char buffer[100];
	time_t theTime = time(0);
 	strftime(buffer, sizeof(buffer), "%a %b %d %Y, %X %Z", localtime(&theTime));
	return buffer;
}

std::string
GeneratorBase::UpperCase(const std::string& arInput)
{
	char ch;
	std::string output(arInput);

	for(std::string::iterator it = output.begin();
		it != output.end();
		++it)
	{
		ch = toupper(*it);
		output.replace(it, it + 1, &ch, 1);
	}
	return output;
}

} // end of the code_sketch namespace
