/* ****************************************************************************
 *  GeneratorBase.h
 *  Copyright (C) 2002, John F. Hubbard
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License (LGPL) along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  As this software is very much in use and evolving, you may want to check
 *  this web site for periodic updates: http://www.hubbard-software.com
 *  You may also post feedback, comments, and (someday) patches there.
 *
 *  My goal is to provide quality, open source programmer tools that are
 *  available to everyone--including commercial users. This is intended to help
 *  the software profession, by providing quality tools that cannot be taken
 *  away from you, no matter where you work.  The LGPL license seems to allow
 *  for all of this, which is why I use it.
 *  --jfh, December, 2002.
 *
 *  $Revision: 1.8 $
 *  $Date: 2002/12/15 01:06:24 $
 *  $Author: jhubbard $
 *************************************************************************** */
#ifndef JFH_CODE_SKETCH_GENERATORBASE_H
#define JFH_CODE_SKETCH_GENERATORBASE_H

namespace code_sketch
{
	 class Guidelines; // Forward declaration
	 class FileCreator; // Forward declaration

/** Purpose: Provide a required interface for code-generating classes; provide
	default implementations of some methods; provide standard implementations
	of other methods.  This is the heart of the shared logic of the code generators.

	Some explanation may be in order.  Every code generating class derives from
	GeneratorBase.  GeneratorBase eliminates code duplication by providing services
	such as SendToFile() and CreateHeader(), each of which depend upon knowing
	something about the child class.  The two things that GeneratorBase wants to
	know (name of the file to be generated, and how to send text to that file) are
	both provided by an instance of FileCreator; each child class has one.

	So it should now be clear that GetFileCreator() represents an obligation that
	the child classes fulfill, while SendToFile() represents a service that the
	parent class provides.  The C++ signatures (one is pure virtual, the other a
	concrete method) reflect this.
 */
class GeneratorBase
{
public:
    /// Constructor.
    GeneratorBase( const Guidelines& arGuidelines );
    virtual ~GeneratorBase();

	// A default implementation is provided for each of the four following
	// code generation methods.
	virtual void CreateHeader();
	virtual void CreateProlog();
	virtual void CreateBody(const std::string& arClassName);
	virtual void CreateEpilog();

	/** GetFileCreator() is pure virtual because things won't work unless each
		derived class provides this hook into child-specific information. */
	virtual FileCreator& GetFileCreator() = 0;

	/** SendToFile() provides a service to the derived classes, which is a
		way to send text to the file that is being generated. */
	std::ostream& 	SendToFile();

	void CreateStandardMakefile();
	void CreateStandard_C_Header( const std::string& arFileName,
			 					  const std::string& arSpecificWords);
	void CreateStandardScriptHeader( const std::string& arFileName,
			 					     const std::string& arSpecificWords);
	std::string MangledUpperCaseName( const Guidelines& arGuidelines );
	std::string DateAndTime();
	std::string UpperCase(const std::string& arInput);

protected:
	const Guidelines & GetGuidelines() const { return mGuidelines; }
		
private:
	// Copying or assignment of this class is prohibited:
    GeneratorBase(const GeneratorBase&);
    GeneratorBase& operator=(const GeneratorBase&);

	void CreateStandardHeader( const std::string& arFileName,
							   const std::string& arSpecificWords,
							   const std::string & arLineStarter);

	const Guidelines & mGuidelines;
};
} // end of the code_sketch namespace

#endif // #ifndef JFH_CODE_SKETCH_GENERATORBASE_H

