/* ****************************************************************************
 *  Guidelines.cpp
 *  Copyright (C) 2002, John F. Hubbard
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License (LGPL) along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  As this software is very much in use and evolving, you may want to check
 *  this web site for periodic updates: http://www.hubbard-software.com
 *  You may also post feedback, comments, and (someday) patches there.
 *
 *  My goal is to provide quality, open source programmer tools that are
 *  available to everyone--including commercial users. This is intended to help
 *  the software profession, by providing quality tools that cannot be taken
 *  away from you, no matter where you work.  The LGPL license seems to allow
 *  for all of this, which is why I use it.
 *  --jfh, December, 2002.
 *
 *  $Revision: 1.16 $
 *  $Date: 2002/12/15 19:45:07 $
 *  $Author: jhubbard $
 *************************************************************************** */
#include <string>
#include <iostream>
#include "Guidelines.h"
#include "FileCreator.h"


namespace code_sketch
{

Guidelines::Guidelines(int argc, char* argv[])
	: CommandLine(argc, argv)
{
	// Setup the vector of class names.
	// This throws a std::string if -class was not passed in on the command line:
	ParseClassList();
	if( ClassList().size() == 0 )
	{
		// If the user did not specify a value for -class, this provides a default:
		mClassList.push_back( "CodeSketchExample" );
	}
	mFirstClassName = ClassList()[0];
}

Guidelines::~Guidelines()
{
}

std::string Guidelines::FirstClassName() const
{
	return mFirstClassName;
}

std::vector<std::string>&  Guidelines::ClassList() const
{
	return mClassList;
}

std::string	Guidelines::ClassHeaderName() const
{
	return (BaseFilename() + ".h");
}

std::string	Guidelines::ClassInlineName() const
{
	return (BaseFilename() + ".icc");
}

std::string	Guidelines::ClassBodyName() const
{
	if( ModeIsCplusplus() )
	{	return BaseFilename() + ".cpp";
	}
	return BaseFilename() + ".c";
}

std::string	Guidelines::MakefileName() const
{
	return "makefile";
}

std::string	Guidelines::DeviceDriverScriptName() const
{
	return std::string("Build") + FirstClassName() + ".bash";
}

std::string	Guidelines::CommonBuildScriptName() const
{
	return "CommonBuildRoutines.bash";
}

std::string	Guidelines::UnitTestName()  const
{
	return (BaseUnitTestName() + ".cpp");
}

std::string	Guidelines::BaseUnitTestName()  const
{
	return (BaseFilename() + "_UnitTest");
}

std::string	Guidelines::Namespace() const
{
	if (Exists("namespace"))
	{	return GetByName("namespace");
	}
	else if (Exists("ns"))
	{	return GetByName("ns");
	}
	return "";
}

std::string	Guidelines::Author() const
{
	if( Exists("author") )
	{
		return GetByName("author");
	}
	return "No author specified";
}

std::string	Guidelines::Copyright() const
{
	if (Exists("copyright"))
	{	return GetByName("copyright");
	}
	else if (Exists("c"))
	{	return GetByName("c");
	}
	// TODO: replace with something like this:
	// return "Copyright (C) 2003, Joe Programmer";
	return "No copyright declared: Auto-generated by CodeSketch (www.hubbard-software.com)";
}

std::string	Guidelines::BaseFilename() const
{
	if (Exists("base_filename"))
	{	return GetByName("base_filename");
	}
	else if (Exists("file"))
	{	return GetByName("file");
	}
	return FirstClassName();
}

std::string Guidelines::OutputDirectory() const
{
	if (Exists("output_dir"))
	{	return GetByName("output_dir");
	}
	else if (Exists("o"))
	{	return GetByName("o");
	}
	return "./";
}

std::string Guidelines::TargetOperatingSystem() const
{
	if( Exists("os") )
	{
		return GetByName("os");
	}
	// Default to Linux, to avoid user frustration.  If someone were to
	// specify -char_driver, and forget to specify -os linux, or or -os lynx,
	// etc., it is better to generate *something* (in this case, a linux device
	// driver), than to fail out with some annoying error.  If the user see
	// code for the wrong operating system, that is a very obvious fix.
	return "linux";
}

std::string Guidelines::CustomInitials() const
{
	if( Exists("initials") )
	{
		return GetByName("initials");
	}
	else if( Exists("in") )
	{
		return GetByName("in");
	}
	return "XXX";
}

std::string Guidelines::LynxOSInfoMainFileName() const
{
    return FirstClassName() + "_CreateInfo.c";
}

std::string Guidelines::LynxOSInfoHeaderFileName() const
{
    return FirstClassName() + "_info.h";
}

std::string Guidelines::LynxOSImportsFileName() const
{
    return FirstClassName() + "_imports.txt";
}

int	Guidelines::FileCreationFlags() const
{
	int flags = 0;

	if (! Exists("quiet") && ! Exists("q"))
	{	flags |= code_sketch::VERBOSE;
	}

	if (Exists("overwrite") || Exists("ow"))
	{	flags |= code_sketch::OVERWRITE;
	}
	return flags;
}

bool Guidelines::ModeIsCplusplus() const
{
	return (!ModeIsAnsiC() && !ModeIsDeviceDriver());
}

bool Guidelines::ModeIsAnsiC() const
{
	return (Exists("ansi_c"));
}

bool Guidelines::ModeIsDeviceDriver() const
{
	return ( IsCharDriver() || IsBlockDriver() );
}

bool Guidelines::IsCharDriver() const
{
	return (Exists("char_driver") || Exists("char"));
}

bool Guidelines::IsBlockDriver() const
{
	return (Exists("block_driver") || Exists("block"));
}
bool Guidelines::SparseCodeGenerationDesired() const
{
	return (Exists("sparse"));
}

bool Guidelines::BuildScriptDesired() const
{
	return (!Exists("no_makefile") && !Exists("no_mf"));
}

bool Guidelines::UnitTestDesired() const
{
	return (!Exists("no_unit_test") && !Exists("no_ut"));
}

bool Guidelines::InlineFileDesired() const
{
	return (!Exists("no_icc"));
}

bool Guidelines::NamespaceDesired() const
{
	return (Exists("namespace")  || Exists("ns"));
}

bool Guidelines::DumpDiagnosticsDesired() const
{
	return (!Exists("no_dump_diagnostics") && !Exists("no_dd"));
}

bool Guidelines::OpenSourceDesired() const
{
	return (Exists("open_source") || Exists("open"));
}

bool Guidelines::SccsKeywordsDesired() const
{
	return ( ! Exists("no_sccs_keywords") && ! Exists("no_sccs"));
}

bool Guidelines::PureInterfaceClassDesired() const
{
	return (Exists("interface") || Exists("i"));
}

bool Guidelines::HelpRequested() const
{
	return ( Exists("h") || Exists("help") );
}

bool Guidelines::VersionReportRequested() const
{
	return ( Exists("version") );
}

bool Guidelines::DiagnosticsReportRequested() const
{
	return ( Exists("diag") || Exists("diagnostics_only") );
}

void
Guidelines::DumpDiagnostics()
{
	using std::endl;
	std::cout <<
	"----------------------Guidelines diagnostics dump----------------" << endl <<
	"Original commands (not necessarily in order: " << endl << AllCommands() << endl <<
	"FirstClassName:            " << FirstClassName() << endl <<
	"ClassHeaderName:           " << ClassHeaderName() << endl <<
	"ClassInlineName:           " << ClassInlineName() << endl <<
	"ClassBodyName:             " << ClassBodyName() << endl <<
	"MakefileName:              " << MakefileName() << endl <<
	"DeviceDriverScriptName:    " << DeviceDriverScriptName() << endl <<
	"UnitTestName:              " << UnitTestName() << endl <<
	"Namespace:                 " << Namespace() << endl <<
	"Author:                    " << Author() << endl <<
	"Copyright:                 " << Copyright() << endl <<
	"BaseFilename:              " << BaseFilename() << endl <<
	"LynxOSInfoMainFileName:    " << LynxOSInfoMainFileName() << endl <<
	"LynxOSInfoHeaderFileName:  " << LynxOSInfoHeaderFileName() << endl <<
	"LynxOSImportsFileName:     " << LynxOSImportsFileName() << endl <<
	"OutputDirectory:           " << OutputDirectory() << endl <<
	"CustomInitials:            " << CustomInitials() << endl <<
	"FileCreationFlags:         " << FileCreationFlags() << endl <<
	"ModeIsCplusplus:           " << ModeIsCplusplus() << endl <<
	"ModeIsAnsiC:               " << ModeIsAnsiC() << endl <<
	"ModeIsDeviceDriver:        " << ModeIsDeviceDriver() << endl <<
	"IsCharDriver:              " << IsCharDriver() << endl <<
	"IsBlockDriver:             " << IsBlockDriver() << endl <<
	"SparseCodeGenerationDesired: " << SparseCodeGenerationDesired() << endl <<
		
	"TargetOperatingSystem:     " << TargetOperatingSystem() << endl <<
	"BuildScriptDesired:        " << BuildScriptDesired() << endl <<
	"UnitTestDesired:           " << UnitTestDesired() << endl <<
	"InlineFileDesired:         " << InlineFileDesired() << endl <<
	"NamespaceDesired:          " << NamespaceDesired() << endl <<
	"DumpDiagnosticsDesired:    " << DumpDiagnosticsDesired() << endl <<
	"OpenSourceDesired:         " << OpenSourceDesired() << endl <<
	"SccsKeywordsDesired:       " << SccsKeywordsDesired() << endl <<
	"PureInterfaceClassDesired: " << PureInterfaceClassDesired() << endl <<
	"Classes to be created:     " << endl;

    for(std::vector<std::string>::iterator it = ClassList().begin();
        it != ClassList().end();
        ++it)
    {
        std::cout << "    " << (*it) << endl;
    }
}

void
Guidelines::ParseClassList()
{
	if( ! Exists("class") )
	{
		return;
	}
	std::string allClasses = GetByName("class");
	const int LENGTH = allClasses.length();
	int wordStart = 0;
	int wordEnd   = 0;
	
	while(wordEnd < LENGTH)
	{
		while(   (allClasses.substr(wordStart, 1) == " "
			      || allClasses.substr(wordStart, 1) == "\t")
				 && wordStart < LENGTH)
		{
			++wordStart;
		}
		if( wordStart > LENGTH - 1)
		{
			if(  mClassList.empty() )
			{
				// Only one word, no inter-word spaces found
				mClassList.push_back( allClasses );
			}
			return; // In any case, there are no more characters to parse
		}
		wordEnd = wordStart;
		while(   (allClasses.substr(wordEnd, 1) != " "
			      && allClasses.substr(wordEnd, 1) != "\t")
			  && wordEnd < LENGTH )
		{
			++wordEnd;
		}
		if( wordEnd < LENGTH )
		{	--wordEnd;
		}
		mClassList.push_back( allClasses.substr(wordStart, wordEnd - wordStart + 1) );
		wordStart = wordEnd + 1;
		wordEnd = wordStart;
	}
}

} // end of the code_sketch namespace
