#ifndef RMI_CONVERSIONS_H
#define RMI_CONVERSIONS_H

#include <string>

namespace RMI {

using std::string;

// If the explicit 
//
//    string convert(const Foo&)
//    template<> Foo convert<Foo>(const string&)
//
// conversion pair is not provided for a Foo class and the class is not derived
// from RMIB, the client-side code (trying to pass arguments of that type or
// trying to return a value of that type) will not compile as the system will
// not know how to convert/retrieve those arguments.

// Convert strings read from the wire to values.
// Always return by-value as there is nothing to return a reference to.

template<class Type> Type convert(const string&);

template<> int    convert<int>    (const string&);
template<> double convert<double> (const string&);
template<> void*  convert<void*>  (const string&);
template<> char*  convert<char*>  (const string&);
template<> string convert<string> (const string&);

// Conversions of the values to strings.
// RMI::Client ALWAYS applies convert(const T&) when converts to RMI::Parameter.
// More so, for user-defined classes
// args should be ALWAYS passed by-reference to prevent copying. Therefore, to
// be consistent we need to maintain the same signature for built-in types.

string convert (void const*  const&);
string convert (void      *  const&);
string convert (char const*  const&);
string convert (char      *  const&);
string convert (int          const&);
string convert (double       const&);
string convert (string       const&);

// string convert (const int*);
// string convert (int*);
//
// Uncomment the two declarations above but do not provide implementation
// for them to prevent compilation of the classes with member functions 
// returning pointers to 'int' or taking such pointers as arguments. Like
// 
//    int* Foo::foo(int*);
// 
// If these declarations are commented out, the instantiation of the 
// generic "template<> string convert<T*>" will be used with T=int for the
// generation of the server-side code. That instantiation will properly
// convert a pointer into its string representation. However, the value is
// unlikely to make any sense in different address space.
//
// As far as the client-side code is concerned, it won't be able to
// reconstruct the pointer from its string representation. 

}

#endif // RMI_CONVERSIONS_H

